/**
 * Copyright (c) 2025 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _HEAVY_CONTEXT_WSTD_FLDR_HPP_
#define _HEAVY_CONTEXT_WSTD_FLDR_HPP_

// object includes
#include "HeavyContext.hpp"
#include "HvSignalEnvelope.h"
#include "HvSignalVar.h"
#include "HvControlBinop.h"
#include "HvControlCast.h"
#include "HvControlVar.h"
#include "HvControlIf.h"
#include "HvSignalLine.h"
#include "HvMath.h"

class Heavy_WSTD_FLDR : public HeavyContext {

 public:
  Heavy_WSTD_FLDR(double sampleRate, int poolKb=10, int inQueueKb=2, int outQueueKb=0);
  ~Heavy_WSTD_FLDR();

  const char *getName() override { return "WSTD_FLDR"; }
  int getNumInputChannels() override { return 2; }
  int getNumOutputChannels() override { return 2; }

  int process(float **inputBuffers, float **outputBuffer, int n) override;
  int processInline(float *inputBuffers, float *outputBuffer, int n) override;
  int processInlineInterleaved(float *inputBuffers, float *outputBuffer, int n) override;

  int getParameterInfo(int index, HvParameterInfo *info) override;
  struct Parameter {
    struct In {
      enum ParameterIn : hv_uint32_t {
        FLDR = 0xE42C892E, // Fldr
        LMTR = 0xCB23DC60, // Lmtr
        MIX = 0x97D46AC6, // Mix
      };
    };
  };

 private:
  HvTable *getTableForHash(hv_uint32_t tableHash) override;
  void scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) override;

  // static sendMessage functions
  static void cBinop_3obXisea_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_RmUMBv07_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_zVU7CUp1_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_BAR7VEgP_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_jGuuCvbr_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_mXjsx0OJ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_94j62L0v_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_Mixz6Ruv_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_iNsxOWUG_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Mmnvdip3_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_BU1oen1e_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_IFHydxI7_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_wZh5ndef_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_l1rDkUO4_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_fmvB9McM_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_OrSkC7CK_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_Z1bZ3huH_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_k1GUl0Nk_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_xCqk2Uzf_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_VDYKoDSp_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_pDoYgH0t_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_8t8mkcCa_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_LIjAHP6K_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_qMjliqMu_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_18v6EXLp_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_XOndzL06_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_3c0OypUx_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_QDsEDgkS_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_mZNTvVEy_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_KFNgFRmd_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_P9gTxjZ2_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_18bEiJ2B_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_FnTtKEHS_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_wXtK9LCb_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_jsF6WzIl_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_cyWtGq7Y_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_6vrM6k6U_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_jr7qxpH2_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_RyXrS68o_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_RcxjTlzE_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_XYQ61ZbM_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_PZbZha3C_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_z0LkF5K4_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_8otWYk6f_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_vR4vg6hv_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_zQyLb1yF_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_JI17lNB4_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_9zquRPPZ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_ZSdQM1Wr_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_o7AS9Swu_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_LA464ryT_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_55HEJ4KH_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cSend_TkUUAEfC_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_AqN9e4xu_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_B9BcYeWV_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_JqlJbJcA_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_TA7E5xxy_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_ab1uF5yw_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_QNkLuaYk_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_Wnvobilh_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_I19iDBjg_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_42sdFc6P_sendMessage(HeavyContextInterface *, int, const HvMessage *);

  // objects
  SignalLine sLine_vwW9tJXN;
  SignalEnvelope sEnv_k1GUl0Nk;
  SignalEnvelope sEnv_xCqk2Uzf;
  SignalEnvelope sEnv_jsF6WzIl;
  SignalEnvelope sEnv_cyWtGq7Y;
  SignalLine sLine_OU010iLu;
  SignalLine sLine_sCWbz2Q5;
  ControlBinop cBinop_3obXisea;
  ControlBinop cBinop_RmUMBv07;
  ControlBinop cBinop_zVU7CUp1;
  ControlBinop cBinop_BAR7VEgP;
  ControlBinop cBinop_94j62L0v;
  ControlBinop cBinop_iNsxOWUG;
  ControlBinop cBinop_Mmnvdip3;
  ControlBinop cBinop_BU1oen1e;
  ControlVar cVar_IFHydxI7;
  ControlVar cVar_l1rDkUO4;
  SignalVarf sVarf_4eMBNX4E;
  ControlVar cVar_fmvB9McM;
  ControlIf cIf_OrSkC7CK;
  ControlIf cIf_Z1bZ3huH;
  ControlBinop cBinop_VDYKoDSp;
  ControlBinop cBinop_8t8mkcCa;
  ControlBinop cBinop_LIjAHP6K;
  ControlBinop cBinop_XOndzL06;
  ControlBinop cBinop_QDsEDgkS;
  SignalVarf sVarf_mFxor9ZJ;
  ControlVar cVar_18bEiJ2B;
  ControlIf cIf_FnTtKEHS;
  ControlIf cIf_wXtK9LCb;
  ControlBinop cBinop_6vrM6k6U;
  ControlBinop cBinop_RyXrS68o;
  ControlBinop cBinop_RcxjTlzE;
  ControlBinop cBinop_z0LkF5K4;
  ControlBinop cBinop_vR4vg6hv;
  ControlVar cVar_ZSdQM1Wr;
  ControlBinop cBinop_o7AS9Swu;
  SignalVarf sVarf_yoRcWKUT;
  SignalVarf sVarf_FlEL0CZT;
  SignalVarf sVarf_EGQVZEmC;
  SignalVarf sVarf_g1GGqIGW;
  ControlBinop cBinop_AqN9e4xu;
};

#endif // _HEAVY_CONTEXT_WSTD_FLDR_HPP_
